# ======================================================================
# NOAA Model Diagnotics Task Force (MDTF) Diagnostic Driver
#
# November 16, 2017  
# Chih-Chieh (Jack) Chen, NCAR, 
# Yi-Hung Kuo, UCLA
#
# This release the following Python modules: 
#     os, glob, json, dataset, numpy, scipy, matplotlib, 
#     networkx, warnings, numba, netcdf4
# ======================================================================
import os
import sys
import glob
import shutil
import subprocess

# ======================================================================
# If using NCL, need to point to it 
# set NCARG_ROOT
os.environ["NCARG_ROOT"]="/home/fwang/ncl"

# ======================================================================
# delete PS
# 1: delete, otherwise not
os.environ["CLEAN"] = "0"

# ======================================================================
# ===     Diagnose transport on T-S plane in the Atlantic Ocean     ====
# Parameters:
#   CMIP5_MODELS: the selected models. at least 2 models 
#   CASENAME    : case name, one of CMIP5_MODELS. User should provide
#     monthly ocean Temperature, Salinity, (volume/mass) transport/velocity
#     files under CASENAME/mon bathymetry and/or layer thickness files under
#     CASENAME/fx bathymetry and/or layer thickness files under CASENAME/fx.
#     Data is from https://esgf-node.llnl.gov/search/cmip5/, historical run. 
#     User should cat monthly files into one, for example:
#     cdo cat so_Omon_CCSM4_historical_r1i1p1_??????-??????.nc CCSM4.so.mon.nc
#   DT,DS       : the resolution of t/s bins
#   LAT0        : the selected latitude 
#   AMOC_3D_Structure: 1 for run, 0 for skip
# ======================================================================`
#CMIP5_MODELS=['ACCESS1-3','bcc-csm1-1-m','CanESM2','CCSM4','CMCC-CESM','CNRM-CM5-2','CSIRO-Mk3-6-0','EC-EARTH','FGOALS-s2','FIO-ESM','GFDL-CM2p1','GFDL-ESM2G','GISS-E2-R','HadGEM2-CC','IPSL-CM5A-MR','MIROC5','MIROC-ESM','MPI-ESM-MR','MRI-CGCM3','NorESM1-M']
CMIP5_MODELS=['CCSM4','GFDL-CM2p1']         # for test

#os.environ["CASENAME"] = 'ACCESS1-3'
#os.environ["CASENAME"] = 'bcc-csm1-1-m'
#os.environ["CASENAME"] = 'CanESM2'
#os.environ["CASENAME"] = "CCSM4"            # for test 1 as in CMIP5_MODELS
#os.environ["CASENAME"] = 'CMCC-CESM'
#os.environ["CASENAME"] = 'CNRM-CM5-2'
#os.environ["CASENAME"] = 'CSIRO-Mk3-6-0'
#os.environ["CASENAME"] = 'EC-EARTH'
#os.environ["CASENAME"] = 'FGOALS-s2'
#os.environ["CASENAME"] = 'FIO-ESM'
os.environ["CASENAME"] = "GFDL-CM2p1"     # for test 2 as in CMIP5_MODELS
#os.environ["CASENAME"] = 'GFDL-ESM2G'
#os.environ["CASENAME"] = 'GISS-E2-R'
#os.environ["CASENAME"] = 'HadGEM2-CC'
#os.environ["CASENAME"] = 'IPSL-CM5A-MR'
#os.environ["CASENAME"] = 'MIROC5'
#os.environ["CASENAME"] = 'MIROC-ESM'
#os.environ["CASENAME"] = 'MPI-ESM-MR'
#os.environ["CASENAME"] = 'MRI-CGCM3'
#os.environ["CASENAME"] = 'NorESM1-M'
os.environ["DT"]     =  "0.2"
os.environ["DS"]     =  "0.04"
os.environ["LAT0"]   =  "26.5"
AMOC_3D_Structure    =  1
# ======================================================================
# ======================================================================


# ======================================================================
#  Home directory for diagnostic code (needs to have 'var_code' and 'obs_data' sub-directories)
os.environ["DIAG_HOME"] = os.getcwd()
os.environ["DIAG_ROOT"] = os.path.dirname(os.environ["DIAG_HOME"])
os.environ["DATA_ROOT"] = os.environ["DIAG_ROOT"]+"/inputdata/"
os.environ["WKDIR"]     = os.getcwd()+"/wkdir"
os.environ["variab_dir"] = os.environ["WKDIR"]+"/MDTF_"+os.environ["CASENAME"]

# ======================================================================
# OUTPUT
# 1 = create tar file of results, tar file in wkdir
os.environ["make_variab_tar"] = "1"

# ======================================================================
# set model. 
# ======================================================================
sys.path.insert(0,'var_code/util/')
os.environ["model"] = "CMIP"
if os.environ["model"] == "CMIP":
   import set_variables_CMIP

# ==================================================================================================
# 1.  Pre-processing for CMIP5_Atlantic_QTS
##     create unified Ocean BASIN index, extract latitude coordinates and dx dy dz
#     This requires model output data. 
# 2.  Project volume transport onto Temperature/Salinity plane
#     AMOC structures on depth, density, temperature and salinity coordinates
#     This requires AMOC_3D_Structure Pre-processing data
#     the code is in NCL
# ==================================================================================================
if AMOC_3D_Structure == 1:
   os.environ["MODELS"] =  ', '.join(CMIP5_MODELS)

   os.environ["SRCDIR"] =  os.environ["DIAG_HOME"]+"/var_code/AMOC_3D_Structure/"
   os.environ["HTMDIR"] =  os.environ["DIAG_HOME"]+"/var_code/html/"
   os.environ["RGBDIR"] =  os.environ["DIAG_HOME"]+"/var_code/util/rgb/"

   os.environ["REFDIR"] =  os.environ["DATA_ROOT"]+"/obs_data/AMOC_3D_Structure/"
   os.environ["CLMREF"] =  os.environ["DATA_ROOT"]+"/obs_data/AMOC_3D_Structure/clim/"
   os.environ["FIXREF"] =  os.environ["DATA_ROOT"]+"/obs_data/AMOC_3D_Structure/fx/"

   os.environ["FIXDIR"] =  os.environ["DATA_ROOT"]+"/"+os.environ["CASENAME"]+"/fx/"
   os.environ["MONDIR"] =  os.environ["DATA_ROOT"]+"/"+os.environ["CASENAME"]+"/mon/"

   os.environ["CASDIR"] =  os.environ["WKDIR"]+"/MDTF_"+os.environ["CASENAME"]+"/"
   os.environ["QTSDIR"] =  os.environ["WKDIR"]+"/MDTF_"+os.environ["CASENAME"]+"/AMOC_3D_Structure/"
   os.environ["PNGDIR"] =  os.environ["WKDIR"]+"/MDTF_"+os.environ["CASENAME"]+"/AMOC_3D_Structure/model/"
   os.environ["FIGDIR"] =  os.environ["WKDIR"]+"/MDTF_"+os.environ["CASENAME"]+"/AMOC_3D_Structure/model/PS/"
   os.environ["OUTDIR"] =  os.environ["WKDIR"]+"/MDTF_"+os.environ["CASENAME"]+"/AMOC_3D_Structure/model/netCDF/"
   os.environ["TMPDIR"] =  os.environ["WKDIR"]+"/MDTF_"+os.environ["CASENAME"]+"/AMOC_3D_Structure/model/netCDF/mon_yr/"
   os.environ["FIGREF"] =  os.environ["WKDIR"]+"/MDTF_"+os.environ["CASENAME"]+"/AMOC_3D_Structure/obs/PS/"
   os.environ["PNGREF"] =  os.environ["WKDIR"]+"/MDTF_"+os.environ["CASENAME"]+"/AMOC_3D_Structure/obs/"
   if not os.path.exists(os.environ["SRCDIR"]):
      os.makedirs(os.environ["SRCDIR"])
   if not os.path.exists(os.environ["CLMREF"]):
      os.makedirs(os.environ["CLMREF"])
   if not os.path.exists(os.environ["FIXREF"]):
      os.makedirs(os.environ["FIXREF"])
   if not os.path.exists(os.environ["FIXDIR"]):
      os.makedirs(os.environ["FIXDIR"])
   if not os.path.exists(os.environ["MONDIR"]):
      os.makedirs(os.environ["MONDIR"])
   if not os.path.exists(os.environ["FIGDIR"]):
      os.makedirs(os.environ["FIGDIR"])
   if not os.path.exists(os.environ["FIGREF"]):
      os.makedirs(os.environ["FIGREF"])
   if not os.path.exists(os.environ["TMPDIR"]):
      os.makedirs(os.environ["TMPDIR"])

   os.system("\cp "+os.environ["RGBDIR"]+"*.rgb "+os.environ["NCARG_ROOT"]+"/lib/ncarg/colormaps/")
   os.system("\cp -f "+os.environ["HTMDIR"]+"mdtf_diag_banner.png "+os.environ["WKDIR"])
   os.system("\cp -f "+os.environ["HTMDIR"]+"mdtf_diag_banner.png "+os.environ["QTSDIR"])
   os.system("\cp -f "+os.environ["HTMDIR"]+"mdtf_diag_banner.png "+os.environ["CASDIR"])
   os.system("\cp -f "+os.environ["HTMDIR"]+"mdtf1.html           "+os.environ["CASDIR"]+"index.html")
   print(os.environ["HTMDIR"]+"mdtf1.html")
   print(os.environ["QTSDIR"]+"index.html")

   ncs=glob.glob(os.environ["MONDIR"]+os.environ["CASENAME"]+"."+os.environ["vmo_var"]+".mon.nc") + \
       glob.glob(os.environ["MONDIR"]+os.environ["CASENAME"]+"."+os.environ["vo_var"] +".mon.nc")
   print(ncs)
   if ncs:
      nc = ncs[-1]
      os.system("python "+os.environ["SRCDIR"]+"basin_lat_dxdydz.py")
      os.system("python "+os.environ["SRCDIR"]+"AMOC_3D_Structure.py")
      os.system("python "+os.environ["SRCDIR"]+"obs_AMOC_3D_Structure.py")
   else:
      print("No eligible nc file exists in"+os.environ["MONDIR"])
      print("AMOC_3D_Structure is NOT Executed as Expected!")
else:
   print("skip AMOC_3D_Structure")

# ======================================================================
#  DO NOT modify: make tar file
# ======================================================================
if os.environ["make_variab_tar"] == "1":
   if os.path.isfile( os.environ["variab_dir"]+".tar" ):
      os.system("mv -f "+os.environ["variab_dir"]+".tar "+os.environ["variab_dir"]+".tar_old")
      os.chdir(os.environ["WKDIR"])
      status = os.system("tar --exclude='*netCDF' --exclude='*nc' --exclude='*ps' --exclude='*PS' -cf MDTF_"+os.environ["CASENAME"]+".tar MDTF_"+os.environ["CASENAME"])
      if not status == 0:
         print("ERROR $0")
         print("trying to do:     tar -cf "+os.environ["variab_dir"]+".tar "+os.environ["variab_dir"])
         exit()
   else:
      os.chdir(os.environ["WKDIR"])
      status = os.system("tar --exclude='*netCDF' --exclude='*nc' --exclude='*ps' --exclude='*PS' -cf MDTF_"+os.environ["CASENAME"]+".tar MDTF_"+os.environ["CASENAME"])
      if not status == 0:
         print("ERROR $0")
         print("trying to do:     tar -cf "+os.environ["variab_dir"]+".tar "+os.environ["variab_dir"])

exit()         
